#!/usr/bin/env python3
# Copyright (C) 2024 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
# Licensed under the GPLv3+ Gnu General Public License
# http://www.gnu.org/licenses/gpl-3.0.html
#
# This script and information about it is maintained at:
# http://lkcl.net/reports/fine_structure_constant/
#
# Last Updated: 25 Dec 2024
#
# http://www.chip-architect.com/news/...
#     ...2004_10_04_The_Electro_Magnetic_coupling_constant.html

from math import pi, e
import math
import bigfloat
try:
    # use bigfloat library, uses Multi-Point Floating Precision library
    from bigfloat import cos, sin, acos, asin, pow, precision, BigFloat, log
    pi = bigfloat.acos(0)*2
    e = bigfloat.exp(1)
    iterations = 70 # paranoid number of iterations
except:
    from math import cos, acos, asin, sin, log
    # bigfloat library not available, use standard python (less accurate)
    BigFloat = float
    class precision: # stub class, does nothing
        def __init__(self, ignore): pass
        def __enter__(self): pass
        def __exit__(self, ignore1, ignore2, ignore3): pass
    iterations = 25 # standard python, pow(2*pi, 70) is too big for float

def alphafunc(g_2_mfactor, mult_factor=2.0):
    with precision(150):
        a = BigFloat(0.007)
        for x in range(1, 30):
            t = 0.0
            g = 0.0
            for i in range(iterations):
                g = g + pow(a, i)/pow(2*pi, t)
                t = t + i
            a = pow(g, 2)/(pow(e, (pow(pi, 2)/mult_factor)))
            a = a * g_2_mfactor
            fsc = 1/a
    return a, g

alpha, g = alphafunc(1.0)

# looking for sin/cos version of alpha
pia = alpha*pi
print("alpha", alpha)
print("pi", pi)
print("pia", pia)
print("alpha^2", pow(alpha,2))

magic=0.000002008616795
cpia  = cos(pia + pi/2)#+magic)
cpiam = cos(-pia + pi/2)
cpiap = cos( pia + pi/2)

print("cpiam", cpiam)
print("cpiap", cpiap)

print()

print(pia)
print(cpia)
print("pia/cpia", pia/cpia)
print("pia/cpia+1", pia/cpia+1)
print("pia/cpia+1", pia/cpia+1)
print("magic", magic)
print("magic/alpha/pi", magic/alpha/pi)
print("magic", 1/magic)
print("magicroot", pow(magic, 0.5))
print("magicroot/alpha", alpha/pow(magic, 0.5))

print()

spia = sin(pia + pi/2)
print("pia", pia)
print("spia", spia)
print("1-spia", 1-spia)
print("pia/spia", pia/spia)
print("magic/1-spia", magic/(1-spia))
print("magic/1-spia/alpha", magic/(1-spia)/alpha)
print("g", g)
gamma = pow(e, -pi*pi/2)
print("gamma", gamma)

print("cpia/spia", cpia/spia)
print("cpia*spia", cpia*spia)

alpha_hypot = pow(1+pow(alpha,2), 0.5)
print ("alphahypot", alpha_hypot)
